"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var Generator_1 = require('../../Generator');
var t = require('./templates');
var addonTemplate_1 = require('./addonTemplate');
var DefaultNanGenerator = (function (_super) {
    __extends(DefaultNanGenerator, _super);
    function DefaultNanGenerator(input) {
        _super.call(this, input);
        this.input = input;
        this.extern_c_functions = [];
        this.methods = [];
        this.inits = [];
    }
    DefaultNanGenerator.prototype.createExternDefinition = function (func) {
        return t.externCFunc(DefaultNanGenerator.allowedTypes, func);
    };
    //TODO: check reserved words such as callback, argv and result
    DefaultNanGenerator.prototype.createMethod = function (func) {
        var parameters = [];
        var externParams = [];
        var deallocations = [];
        var v8ReturnValue;
        func.parameters.forEach(function (param, index) {
            switch (param.type) {
                case "c_int":
                case "c_double":
                case "c_float":
                case "bool":
                    parameters.push(t.funcParameterDefault(index, param));
                    break;
                case "*c_char":
                    parameters.push(t.funcParameterCString(index, param));
                    deallocations.push(t.dealloc(param));
                    break;
                default:
                    Generator_1.l.err(param.name + " param " + param.type);
                    throw Error(Generator_1.Generator.UNSUPPORTED_TYPE);
            }
            externParams.push(param.name);
        });
        switch (func.return) {
            case "c_int":
            case "c_double":
            case "c_float":
            case "bool":
                v8ReturnValue = t.returnValueDefault();
                break;
            case "*c_char":
                v8ReturnValue = t.returnValueCString();
                break;
            case "void":
                break;
            default:
                Generator_1.l.err(func.name + " returns " + func.return);
                throw Error(Generator_1.Generator.UNSUPPORTED_TYPE);
        }
        return t.nanMethod(func, parameters, externParams, deallocations, v8ReturnValue);
    };
    DefaultNanGenerator.prototype.createInit = function (func) {
        return t.init(func);
    };
    DefaultNanGenerator.prototype.generate = function () {
        var _this = this;
        this.input.functions.forEach(function (func) {
            _this.extern_c_functions.push(_this.createExternDefinition(func));
            _this.methods.push(_this.createMethod(func));
            _this.inits.push(_this.createInit(func));
        });
        return addonTemplate_1.addonTemplate(this.extern_c_functions.join(''), this.methods.join(''), this.inits.join(''));
    };
    ;
    DefaultNanGenerator.allowedTypes = [
        /* some primitives from Rust's libc */
        //"c_double", //JS number
        "c_int",
        "c_float",
        "c_double",
        "bool",
        "void",
        /* pointers */
        "*c_char",
    ];
    return DefaultNanGenerator;
}(Generator_1.Generator));
exports.DefaultNanGenerator = DefaultNanGenerator;
