"use strict";
var Generator_1 = require('../../Generator');
var os = require('os');
function externCFunc(allowedTypes, func) {
    var parameters = [];
    func.parameters.forEach(function (param) {
        if (allowedTypes.indexOf(param.type) < 0) {
            Generator_1.l.err(param.type + " param " + param.type);
            throw Error(Generator_1.Generator.UNSUPPORTED_TYPE);
        }
        parameters.push(Generator_1.Generator.mapToCType(param.type) + " " + param.name);
    });
    if (allowedTypes.indexOf(func.return) < 0) {
        Generator_1.l.err(func.name + " returns " + func.return);
        throw Error(Generator_1.Generator.UNSUPPORTED_TYPE);
    }
    return "\n  extern \"C\" " + Generator_1.Generator.mapToCType(func.return) + " " + func.name + "(" + parameters.join(", ") + ");";
}
exports.externCFunc = externCFunc;
function funcParameterDefault(index, param) {
    var inType = Generator_1.Generator.mapToCType(param.type);
    if (param.type == "c_float") {
        Generator_1.l.warn(Generator_1.Generator.WARN_FLOAT);
        inType = "double";
    }
    return "\n  " + inType + " " + param.name + " = To<" + inType + ">(info[" + index + "]).FromJust();";
}
exports.funcParameterDefault = funcParameterDefault;
function funcParameterCString(index, param) {
    return "\n  Nan::HandleScope scope;\n  String::Utf8Value cmd_" + param.name + "(info[" + index + "]);\n  string s_" + param.name + " = string(*cmd_" + param.name + ");\n  char *" + param.name + " = (char*) malloc (s_" + param.name + ".length() + 1);\n  strcpy(" + param.name + ", s_" + param.name + ".c_str());";
}
exports.funcParameterCString = funcParameterCString;
function dealloc(param) {
    return "\n  free(" + param.name + ");";
}
exports.dealloc = dealloc;
function returnValueDefault() {
    return "\n  info.GetReturnValue().Set(result);";
}
exports.returnValueDefault = returnValueDefault;
function returnValueCString() {
    return "\n  info.GetReturnValue().Set(Nan::New<String>(result).ToLocalChecked());\n  free(result);";
}
exports.returnValueCString = returnValueCString;
function init(func) {
    return "\n  Nan::Set(\n    target, New(\"" + func.name + "\").ToLocalChecked(),\n    Nan::GetFunction(New<FunctionTemplate>(" + func.name + ")).ToLocalChecked()\n  );";
}
exports.init = init;
function v8asyncOutput(returnType) {
    switch (returnType) {
        case "c_int":
        case "c_double":
        case "c_float":
        case "bool":
            return "Nan::New(result)";
        case "*c_char":
        default:
            return "Nan::New(result).ToLocalChecked()";
    }
}
function asyncWorker(func) {
    var workerParams = func.parameters.map(function (p) {
        return { name: p.name, type: Generator_1.Generator.mapToCType(p.type) };
    });
    return "\nclass " + func.name + "Worker : public AsyncWorker {\n public:\n  " + func.name + "Worker(Callback *callback, " + workerParams.map(function (p) { return (p.type + " " + p.name); }).join(', ') + ")\n    : AsyncWorker(callback), " + workerParams.map(function (p) { return (p.name + "(" + p.name + ")"); }).join(', ') + " {}\n  ~" + func.name + "Worker() {}\n\n  // Executed inside the worker-thread.\n  // It is not safe to access V8, or V8 data structures\n  // here, so everything we need for input and output\n  // should go on 'this'.\n  void Execute () {\n    result = " + func.name + "(" + workerParams.map(function (p) { return p.name; }).join(', ') + ");\n  }\n\n  // Executed when the async work is complete\n  // this function will be run inside the main event loop\n  // so it is safe to use V8 again\n  void HandleOKCallback () {\n    Nan::HandleScope scope;\n    Local<Value> argv[] = {\n      " + v8asyncOutput(func.return) + "\n    };\n    callback->Call(1, argv);\n  }\n\n private:\n  " + workerParams.map(function (p) { return (p.type + " " + p.name + ";"); }).join(os.EOL) + "\n  " + Generator_1.Generator.mapToCType(func.return) + " result;\n};\n";
}
function nanMethod(func, parameters, externParams, deallocations, v8ReturnValue) {
    var funcCall;
    if (func.async) {
        funcCall = "\n  Nan::Callback * callback = new Nan::Callback(info[" + externParams.length + "].As<v8::Function>());\n  AsyncQueueWorker(new " + func.name + "Worker(callback, " + externParams.join(", ") + "));";
        v8ReturnValue = '';
    }
    else {
        funcCall = Generator_1.Generator.mapToCType(func.return) + " result = " + func.name + "(" + externParams.join(", ") + ");";
    }
    return (" " + (func.async ? asyncWorker(func) : '') + "\nNAN_METHOD(" + func.name + ") {") +
        (parameters.join("") + "\n  " + funcCall) +
        ("" + v8ReturnValue) +
        (deallocations.join('') + "\n}");
}
exports.nanMethod = nanMethod;
